<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\Exportable;
use PhpOffice\PhpSpreadsheet\Style\Border;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithMapping;
use Maatwebsite\Excel\Concerns\WithDrawings;
use Maatwebsite\Excel\Concerns\WithHeadings;
use PhpOffice\PhpSpreadsheet\Style\Alignment;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithProperties;
use PhpOffice\PhpSpreadsheet\Worksheet\Drawing;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Concerns\WithCustomStartCell;

class SalesExport implements
    FromCollection,
    WithHeadings,
    WithMapping,
    WithStyles,
    // WithProperties
    WithCustomStartCell
{
    use Exportable;

    protected $sale;
    protected $order_taxes;

    public function __construct($sale, $order_taxes)
    {
        $this->sale = $sale;
        $this->order_taxes = $order_taxes;
    }

    public function collection()
    {
        return collect([$this->sale]);
    }
    public function map($sale): array
    {
        return [
            // $sale->transaction_date,
            // $sale->transaction_date,
            // $sale->total_before_tax,
            // $sale->payment_status,
            // $sale->location_id,
        ];
    }

    public function startCell(): string
    {
        return 'B11';
    }

    public function styles(Worksheet $sheet)
    {

        $sheet->mergeCells("B4:C4");
        $sheet->mergeCells("B3:C3");

        // Set horizontal and vertical alignment for merged cells B3, B4, C3, and C4
        // $sheet->getStyle("B3:C3")->getAlignment()->setVertical(Alignment::VERTICAL_CENTER);

        $sheet->mergeCells("B2:C2");
        $sheet->getRowDimension('11')->setRowHeight(20);
        $sheet->getColumnDimension('B')->setWidth(15);
        $sheet->getColumnDimension('C')->setWidth(15);
        $sheet->getColumnDimension('D')->setWidth(15);
        $sheet->getColumnDimension('E')->setWidth(30);
        $sheet->getColumnDimension('H')->setWidth(15);
        $sheet->getColumnDimension('I')->setWidth(15);
        $sheet->getColumnDimension('J')->setWidth(20);
        $sheet->getColumnDimension('K')->setWidth(20);
        $sheet->getColumnDimension('L')->setWidth(25);
        $sheet->getColumnDimension('M')->setWidth(40);

        $sheet->setShowGridlines(false);

        $sheet->setCellValue('B2', 'EXECUTIVE OPTICAL, INC:');
        $sheet->setCellValue('E2', now()->format('m/d/Y'));
        $sheet->setCellValue('B3', '200-068-519-000');
        $sheet->setCellValue('B4', 'Execut ive Optical, Inc.');
        $sheet->getStyle("B3")->getAlignment()->setHorizontal(Alignment::HORIZONTAL_RIGHT);



        // $sheet->setCellValue('C4', 'ive Optical, Inc.');
        $sheet->setCellValue('D4', $this->sale->invoice_no);
        $sheet->mergeCells("B5:D6");

        // Set the cell value to the desired text and wrap the text
        $sheet->setCellValue("B5", "6th Flr. Cedar Executive Bldg. 26 timog Ave. Cor. Scout Tobias brgy Laging Handa Quezon City");
        $sheet->getStyle("B5")->getAlignment()->setWrapText(true);
        $sheet->getStyle("B5")->getFont()->setSize('9');

        $sheet->setCellValue('E5', '30 DAYS');
        $sheet->getStyle("B2")->getAlignment()->setHorizontal(Alignment::HORIZONTAL_LEFT);
        $sheet->getStyle("D4")->getAlignment()->setHorizontal(Alignment::HORIZONTAL_RIGHT);
        $sheet->getStyle("B2")->getFont()->setName('Century Gothic');
        $sheet->getStyle("C2")->getFont()->setName('Century Gothic');
        $sheet->getStyle("B2")->getFont()->setBold(true);
        $sheet->getStyle("C2")->getFont()->setBold(true);

        // Set font color for cell B2 and C2

        $sheet->getStyle("B2")->getFont()->setSize('10.5');
        $sheet->getStyle("C2")->getFont()->setSize('10.5');

        $currentRow = 10;
        $lastRow = 0;

        foreach ($this->sale->sell_lines as $key => $sell) {
            // Set the row number for the current product
            $row = $currentRow;

            // Set the values for the current product
            $sheet->setCellValue("A{$row}", $sell->quantity);
            $sheet->setCellValue("B{$row}", 'Unit');
            $sheet->setCellValue("C{$row}", $sell->product->name);
            $sheet->setCellValue("D{$row}", $sell->product->unit_price_before_discount);
            $sheet->setCellValue("E{$row}", $sell->quantity * $sell->unit_price_inc_tax);

            $sheet->getColumnDimension('C')->setWidth(60);

            $sheet->getStyle("B{$row}:N{$row}")->getAlignment()->setVertical(Alignment::VERTICAL_CENTER);
            $sheet->getStyle("B{$row}:N{$row}")->getAlignment()->setHorizontal(Alignment::HORIZONTAL_CENTER);

            // Increment the current row for the next product
            $currentRow++;

            // Calculate the last row for the current set of products
            $lastRow = $currentRow - 1;

            // Add additional styling or borders if needed
            // $sheet->getStyle("B{$lastRow}:N{$lastRow}")->getBorders()->getBottom()->setBorderStyle(Border::BORDER_DOUBLE);
        }
        // Calculate the tax (customize this based on your business logic)
        $subTotal = $this->sale->total_before_tax; // 10% tax rate for example

        // Calculate the tax based on your business logic
        $tax = 0;
        if (!empty($order_taxes)) {
            foreach ($order_taxes as $taxAmount) {
                $tax += $taxAmount;
            }
        }

        // Calculate the total amount after VAT
        $totalAmount = $subTotal + $tax;
        $currentRow = $currentRow + 10;
        // Set the calculated values in the E column
        $sheet->setCellValue("E{$currentRow}",  $subTotal);
        $sheet->getStyle("E{$currentRow}")->getAlignment()->setHorizontal(Alignment::HORIZONTAL_CENTER);
        $sheet->setCellValue("E" . ($currentRow + 3),  $tax);
        $sheet->getStyle("E" . ($currentRow + 3))->getAlignment()->setHorizontal(Alignment::HORIZONTAL_CENTER);

        $sheet->setCellValue("E" . ($currentRow + 5), $totalAmount);
        $sheet->getStyle("E" . ($currentRow + 5))->getAlignment()->setHorizontal(Alignment::HORIZONTAL_CENTER);
    }

    public function headings(): array
    {
        return [];
    }
}
